@@ -3,25 +3,34 @@ import numpy as np
 import cv2
 from time import time
 
+import pandas as pd
 from imutils.video import FPS
+import numpy as np
 import argparse
 import imutils
+import cv2
 
 class PigDetection:
     """
     Class implements Yolo5 model to make inferences on a youtube video using Opencv2.
     """
 
-    def __init__(self, capture_index, model_name, ratio=0.2):
+    def __init__(self, capture_index, model_name, ratio=0.2, distance_between_pixel=5):
         """
         Initializes the class with youtube url and output file.
         :param url: Has to be as youtube URL,on which prediction is made.
         :param out_file: A valid output file name.
         """
         self.capture_index = capture_index
-        self.model_name = "model/best.pt"
         self.model = self.load_model(model_name)
         self.ratio = ratio
+        self.n = 0
+        self.round = 1
+        self.data = [0, 0]
+        self.data_interest = [0, 0] 
+        self.pre_frame = 0
+        self.post_frame = 0
+        self.distance_between_pixel = distance_between_pixel
         self.classes = ["Other", "Sleep", "Stand"]
         self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
         print("Using Device: ", self.device)
@@ -42,8 +51,7 @@ class PigDetection:
         if model_name:
             model = torch.hub.load('ultralytics/yolov5', 'custom', path=model_name, force_reload=True)
         else:
-            model = torch.hub.load('ultralytics/yolov5', 'custom', path=model_name, force_reload=True)
-            #model = torch.hub.load('ultralytics/yolov5', 'yolov5s', pretrained=True)
+            model = torch.hub.load('ultralytics/yolov5', 'yolov5s', pretrained=True)
         return model
 
     def score_frame(self, frame):
@@ -93,71 +101,62 @@ class PigDetection:
                 cv2.putText(frame, self.class_to_label(labels[i]), (x1, y1), cv2.FONT_HERSHEY_SIMPLEX, 0.9, bgr, 2)
 
         return frame
+    
+    def create_data(self, frame):
+        result = self.model(frame).pandas().xyxy[0]
+        result['n'] = np.arange(0, result.shape[0])
+        if self.round < 3:
+            self.pre_frame = result
+            self.post_frame = result
+            self.round += 1
+        elif self.round > 2 & result.shape[0] > 1:
+            self.pre_frame = self.post_frame
+            self.post_frame = result
+            self.round += 1
+        self.data[0] = self.pre_frame
+        self.data[1] = self.post_frame
+
+        self.data_interest[0] = self.data[self.n]
+        self.data_interest[1] = self.data[self.n+1]
+        #print(self.data_interest[0].shape)
+        #print(self.data_interest[0]['xmin'].iloc[0])
+        for i in range(self.data_interest[0].shape[0]):
+           for j in range(self.data_interest[1].shape[0]):
+              if abs(self.data_interest[0]['xmin'].iloc[i] - self.data_interest[1]['xmin'].iloc[j]) < self.distance_between_pixel:
+                  if self.data_interest[0]['n'].iloc[i] != self.data_interest[1]['n'].iloc[j]:
+                      self.data_interest[1]['n'].iloc[j] = self.data_interest[0]['n'].iloc[i]
+        #print(self.data)
+        #print("data_interest[0]: ", data_interest[0])
+        return self.data_interest[1]
 
     def detect(self):
-        """
-        This function is called when class is executed, it runs the loop to read the video frame by frame,
-        and write the output into a new file.
-        :return: void
-        """
-        #cap = self.get_video_capture()
-        #assert cap.isOpened()
         stream = self.get_video_capture()
         assert stream.isOpened()
         fps = FPS().start()
-
+        n = False
         while True:
-          
-            #ret, frame = cap.read()
-            #assert ret
-            
-            #frame = cv2.resize(frame, (416,416))
-            
-            #start_time = time()
-            #results = self.score_frame(frame)
-            #frame = self.plot_boxes(results, frame)
-            
-            #end_time = time()
-            #fps = 1/np.round(end_time - start_time, 2)
-            #print(f"Frames Per Second : {fps}")
-             
-            #cv2.putText(frame, f'FPS: {int(fps)}', (20,20), cv2.FONT_HERSHEY_SIMPLEX, 0.7, (0,255,0), 2)
-            
-            #cv2.imshow("PigDectection", frame)
- 
-            #if cv2.waitKey(1) & 0xFF == 27:
-            #    break
-
-        #cap.release()
             (grabbed, frame) = stream.read()
-        
-           #results = self.score_frame(frame)
-            #frame = self.plot_boxes(results, frame)
-            #frame = self.plot_boxes(results, frame)
             frame = imutils.resize(frame, width=450)
             frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
             frame = np.dstack([frame, frame, frame])
-            #frame = cv2.resize(frame, (416,416))
-            
             start_time = time()
+
+            data = self.create_data(frame)
+            #if len(data) > 1:
+                #print(data)
             results = self.score_frame(frame)
+            
             frame = self.plot_boxes(results, frame)
             
             end_time = time()
-            #fps = 1/np.round(end_time - start_time, 2)
-            #print(f"Frames Per Second : {fps}")
-             
+            for i in range(len(data)): #str(data['n'].iloc[i])
+                cv2.putText(frame, str(data['n'].iloc[i]), (int(data['xmax'].iloc[i]), int(data['ymin'].iloc[i])), cv2.FONT_HERSHEY_SIMPLEX, 0.7, (0,0,255), 2)
+                #print(data['xmin'].iloc[i])
             #cv2.putText(frame, f'FPS: {int(fps)}', (20,20), cv2.FONT_HERSHEY_SIMPLEX, 0.7, (0,255,0), 2)
             if cv2.waitKey(1) & 0xFF == ord('q'):
                 break
             cv2.imshow("Frame", frame)
             cv2.waitKey(1)
             fps.update()
-            #if cv2.waitKey(1) & 0xFF == 27:
-                #break
       
         stream.release()
-
-#print("detect...")
-#detection = PigDetection("C:/Users/tatsa/Desktop/PIG_PROJECT_AJ.TARN/Video/results_dataset/VDOแม่หมู/pig_4.mp4", "C:/Users/tatsa/Desktop/PIG_PROJECT_AJ.TARN/Chula_2/best_10eps.pt")
-#detection.detect()
\ No newline at end of file
